/* acts as an echo server.  "echo" as in tcp echo, not icmp
   echo.  That is, whatever it reads as input on a newly
   accepted socket is immediately written back, until that
   socket is eventually closed. */

#include <stdlib.h>
#include <sys/time.h>
#include <stdio.h>
#include <unistd.h>
#include "amphibian_app.h"
#include "fishperf.h"

#define min(x,y) ({ \
 const typeof(x) _xi = (x);       \
 const typeof(y) _yi = (y);       \
 (void) (&_xi == &_yi);           \
 _xi < _yi ? _xi : _yi; })
#define max(x,y) ({ \
 const typeof(x) _xa = (x);       \
 const typeof(y) _ya = (y);       \
 (void) (&_xa == &_ya);           \
 _xa > _ya ? _xa : _ya; })

int main(int argc, const char *argv[]) {
  struct sockaddr_fish sf;
  struct sockaddr_fish remote;
  socklen_t sl = sizeof(struct sockaddr_fish);
  int csk, ask = fishsockets_socket(AF_FISH, SOCK_STREAM, 0);
  remote.sfish_family = AF_FISH;
  sf.sfish_family = AF_FISH;
  if(argc > 1) {
    sf.sfish_port = FISH_SERVICE_ECHO;
  } else {
    sf.sfish_port = FISH_SERVICE_DISCARD;
  }
  fishsockets_bind(ask, (struct sockaddr *)&sf, sl);
  fprintf(stderr, "server: bound\n");
  while((csk = fishsockets_accept(ask, (struct sockaddr *)&remote, &sl)) > 0) {
#define BUFSIZE 4096
    char buf[BUFSIZE];
    int len, transferred;
    double deltat;
    struct timeval tv_start,tv_end;
    gettimeofday(&tv_start, NULL);
    memset(buf, 0, BUFSIZE); /* clear the buffer so that it is null terminated. */
    for(transferred = 0; 
        (len = fishsockets_read(csk, buf, BUFSIZE)) > 0;
        transferred += len ) {
      if(buf[0] != char_from_index(transferred)) {
        int i;
        fprintf(stderr, "server: read an unexpected byte (not %x)\n",
                char_from_index(transferred));
        for(i=0; i< min(len,20); i++) {
          fprintf(stderr, "%0x ", buf[i]);
        }
        fprintf(stderr, "\n");
      } else {
        // fprintf(stderr, "server: read the right byte\n");
      }
      if(sf.sfish_port == FISH_SERVICE_ECHO) {
        if(fishsockets_write(csk, buf, len) < 0) {
          fprintf(stderr, "server: error writing");
        }
      } /* else, was discard, do nothing */
      memset(buf, 0, 512);
    }
    if(len < 0) {
      fprintf(stderr, "server: error reading");
    }
    fishsockets_close(csk);
    gettimeofday(&tv_end, NULL);
    deltat = (double)(tv_end.tv_sec - tv_start.tv_sec) + 
      (double)(tv_end.tv_usec - tv_start.tv_usec)/1000000.0;
    printf("server: read %d bytes in %5.3f seconds, at %5.3f megabits per second\n", 
           transferred, deltat, (float)transferred/deltat/1000000*8.0);
  }
  exit(EXIT_SUCCESS);
}
